/**
* @file
* Command Class Firmware Update source file.
* @copyright 2018 Silicon Laboratories Inc.
*/

/****************************************************************************/
/*                              INCLUDE FILES                               */
/****************************************************************************/
#include <CC_FirmwareUpdate.h>
#include <ZW_TransportLayer.h>
#include <misc.h>
#include <ota_util.h>
#include <CRC.h>
#include <CC_Version.h>
#include <string.h>
#include <CC_Supervision.h>
#include <CC_ManufacturerSpecific.h>
#ifdef EFR32FG
#include<api/btl_interface.h>
#endif

//#define DEBUGPRINT
#include "DebugPrint.h"
/****************************************************************************/
/*                      PRIVATE TYPES and DEFINITIONS                       */
/****************************************************************************/


/****************************************************************************/
/*                              PRIVATE DATA                                */
/****************************************************************************/

/****************************************************************************/
/*                              EXPORTED DATA                               */
/****************************************************************************/

/****************************************************************************/
/*                            PRIVATE FUNCTIONS                             */
/****************************************************************************/

received_frame_status_t
handleCommandClassFWUpdate(
  RECEIVE_OPTIONS_TYPE_EX *rxOpt,
  ZW_APPLICATION_TX_BUFFER *pCmd,
  uint8_t cmdLength)
{
  if (true == Check_not_legal_response_job(rxOpt))
  {
    // None of the following commands support endpoint bit addressing.
    return RECEIVED_FRAME_STATUS_FAIL;
  }

  switch (pCmd->ZW_Common.cmd)
  {
    case FIRMWARE_MD_GET_V5:
    {
      ZAF_TRANSPORT_TX_BUFFER  TxBuf;
      ZW_APPLICATION_TX_BUFFER* pTxBuf = &(TxBuf.appTxBuf);
      memset((uint8_t*)pTxBuf, 0, sizeof(ZW_APPLICATION_TX_BUFFER) );

      uint8_t i;
      uint8_t * pData;
      TRANSMIT_OPTIONS_TYPE_SINGLE_EX * pTxOptionsEx;
      RxToTxOptions(rxOpt, &pTxOptionsEx);

      uint16_t manufacturerID = 0;
      uint16_t productID      = 0;
      CC_ManufacturerSpecific_ManufacturerSpecificGet_handler(&manufacturerID, &productID);

      pTxBuf->ZW_FirmwareMdReport1byteV5Frame.cmdClass = COMMAND_CLASS_FIRMWARE_UPDATE_MD_V5;
      pTxBuf->ZW_FirmwareMdReport1byteV5Frame.cmd = FIRMWARE_MD_REPORT_V5;
      pTxBuf->ZW_FirmwareMdReport1byteV5Frame.manufacturerId1 = (uint8_t)(manufacturerID >> 8);
      pTxBuf->ZW_FirmwareMdReport1byteV5Frame.manufacturerId2 = (uint8_t)(manufacturerID & 0xFF);
      pTxBuf->ZW_FirmwareMdReport1byteV5Frame.firmware0Id1 = (uint8_t)(handleFirmWareIdGet(0) >> 8);
      pTxBuf->ZW_FirmwareMdReport1byteV5Frame.firmware0Id2 = (uint8_t)(handleFirmWareIdGet(0) & 0xff);
      pTxBuf->ZW_FirmwareMdReport1byteV5Frame.firmware0Checksum1 = 0x00; // Checksum N/A for SDK7
      pTxBuf->ZW_FirmwareMdReport1byteV5Frame.firmware0Checksum2 = 0x00; // Checksum N/A for SDK7
      pTxBuf->ZW_FirmwareMdReport1byteV5Frame.firmwareUpgradable = 0xFF; // Hardcode to upgradable.
      pTxBuf->ZW_FirmwareMdReport1byteV5Frame.numberOfFirmwareTargets = handleNbrFirmwareVersions() - 1;
      pTxBuf->ZW_FirmwareMdReport1byteV5Frame.maxFragmentSize1 = (handleCommandClassFirmwareUpdateMaxFragmentSize() >> 8);
      pTxBuf->ZW_FirmwareMdReport1byteV5Frame.maxFragmentSize2 = (handleCommandClassFirmwareUpdateMaxFragmentSize() & 0xFF);
      pData = (uint8_t *)pTxBuf;
      for (i = 1; i < handleNbrFirmwareVersions(); i++)
      {
        *(pData + 10 + (2 * i)) = (uint8_t)(handleFirmWareIdGet(i) >> 8);
        *(pData + 10 + (2 * i) + 1) = (uint8_t)(handleFirmWareIdGet(i) & 0xff);
      }
      // Fill out the hardwareVersion field, located right after the 0-n extra firmwareId fields
      *(pData + 10 + (2 * i)) = CC_Version_GetHardwareVersion_handler();

      if ( EQUEUENOTIFYING_STATUS_SUCCESS != Transport_SendResponseEP(
             (uint8_t *)pTxBuf,
             10 + (2 * i) + 1,
             pTxOptionsEx,
             NULL))
      {
        /*Job failed */
        ;
      }
      return RECEIVED_FRAME_STATUS_SUCCESS;
    }
    return RECEIVED_FRAME_STATUS_FAIL;
    break;
    case FIRMWARE_UPDATE_MD_REPORT_V5:
      {
        uint16_t crc16Result = CRC_CheckCrc16(0x1D0F, &(pCmd->ZW_Common.cmdClass), cmdLength);
        uint16_t  firmwareUpdateReportNumber = ((uint16_t)(pCmd->ZW_FirmwareUpdateMdReport1byteV5Frame.properties1 &
                                      FIRMWARE_UPDATE_MD_REPORT_PROPERTIES1_REPORT_NUMBER_1_MASK_V5) << 8) +
                                     (uint16_t)(pCmd->ZW_FirmwareUpdateMdReport1byteV5Frame.reportNumber2);
        uint8_t fw_actualFrameSize =  cmdLength -
                                  /* Calculate length of actual data1 field */
                                  (sizeof(pCmd->ZW_FirmwareUpdateMdReport1byteV5Frame.cmdClass) +
                                   sizeof(pCmd->ZW_FirmwareUpdateMdReport1byteV5Frame.cmd) +
                                   sizeof(pCmd->ZW_FirmwareUpdateMdReport1byteV5Frame.properties1) +
                                   sizeof(pCmd->ZW_FirmwareUpdateMdReport1byteV5Frame.reportNumber2) +
                                   sizeof(pCmd->ZW_FirmwareUpdateMdReport1byteV5Frame.checksum1) +
                                   sizeof(pCmd->ZW_FirmwareUpdateMdReport1byteV5Frame.checksum2));


        handleCmdClassFirmwareUpdateMdReport(crc16Result,
                                             firmwareUpdateReportNumber,
                                             pCmd->ZW_FirmwareUpdateMdReport1byteV5Frame.properties1,
                                             &(pCmd->ZW_FirmwareUpdateMdReport1byteV5Frame.data1),
                                             fw_actualFrameSize);


      }
      return RECEIVED_FRAME_STATUS_SUCCESS;
      break;
    case FIRMWARE_UPDATE_MD_REQUEST_GET_V5:
      {
        ZAF_TRANSPORT_TX_BUFFER  TxBuf;
        ZW_APPLICATION_TX_BUFFER* pTxBuf = &(TxBuf.appTxBuf);
        memset((uint8_t*)pTxBuf, 0, sizeof(ZW_APPLICATION_TX_BUFFER) );
        DPRINT("a");

        uint8_t fwTarget = 0;
        uint16_t fragmentSize = 0;

        TRANSMIT_OPTIONS_TYPE_SINGLE_EX * pTxOptionsEx;
        DPRINT("b");
        RxToTxOptions(rxOpt, &pTxOptionsEx);

        /*Get max size for V4 commands*/
        fragmentSize = handleCommandClassFirmwareUpdateMaxFragmentSize();

        /* Verify if the FragmentSize and FirmwareTarget fields are part of the command (V3 and onwards) */
        if(sizeof(ZW_FIRMWARE_UPDATE_MD_REQUEST_GET_V3_FRAME) <= cmdLength)
        {
          DPRINT("c");
          fwTarget = pCmd->ZW_FirmwareUpdateMdRequestGetV5Frame.firmwareTarget;
          fragmentSize =  (((uint16_t)pCmd->ZW_FirmwareUpdateMdRequestGetV5Frame.fragmentSize1) << 8);
          fragmentSize +=  (((uint16_t)pCmd->ZW_FirmwareUpdateMdRequestGetV5Frame.fragmentSize2) & 0xff);
        }

        pTxBuf->ZW_FirmwareUpdateMdRequestReportV5Frame.cmdClass = COMMAND_CLASS_FIRMWARE_UPDATE_MD_V5;
        pTxBuf->ZW_FirmwareUpdateMdRequestReportV5Frame.cmd = FIRMWARE_UPDATE_MD_REQUEST_REPORT_V5;

        /* Validate the hardwareVersion field (V5 and onwards) */
        uint8_t hardwareVersion = CC_Version_GetHardwareVersion_handler();
        if ( (sizeof(ZW_FIRMWARE_UPDATE_MD_REQUEST_GET_V5_FRAME) <= cmdLength) &&
            (hardwareVersion != pCmd->ZW_FirmwareUpdateMdRequestGetV5Frame.hardwareVersion) )
        {
          DPRINT("h");
          /* Invalid hardware version */
          pTxBuf->ZW_FirmwareUpdateMdRequestReportV5Frame.status = FIRMWARE_UPDATE_MD_REQUEST_REPORT_INVALID_HARDWARE_VERSION_V5;
        }
        else if ( fwTarget < handleNbrFirmwareVersions())
        {
          uint8_t * pData = (uint8_t *)&(pCmd->ZW_FirmwareUpdateMdRequestGetV5Frame.manufacturerId1);
          FW_UPDATE_GET fwUpdateData;
          DPRINT("d");

          fwUpdateData.manufacturerId = (((uint16_t)*(pData + 0)) << 8) | *(pData + 1);
          fwUpdateData.firmwareId     = (((uint16_t)*(pData + 2)) << 8) | *(pData + 3);
          fwUpdateData.checksum       = (((uint16_t)*(pData + 4)) << 8) | *(pData + 5);

          handleCmdClassFirmwareUpdateMdReqGet(
              rxOpt,
              fwTarget,
              fragmentSize,
              &fwUpdateData,
              &(pTxBuf->ZW_FirmwareUpdateMdRequestReportV5Frame.status));
        }
        else
        {
          DPRINT("e");
          /*wrong target!!*/
          pTxBuf->ZW_FirmwareUpdateMdRequestReportV5Frame.status = FIRMWARE_UPDATE_MD_REQUEST_REPORT_NOT_UPGRADABLE_V5;
        }

        if (EQUEUENOTIFYING_STATUS_SUCCESS != Transport_SendResponseEP(
                (uint8_t *)pTxBuf,
                sizeof(ZW_FIRMWARE_UPDATE_MD_REQUEST_REPORT_V5_FRAME),
                pTxOptionsEx,
		        ZCB_CmdClassFwUpdateMdReqReport))
        {
          DPRINT("f");
          ZCB_CmdClassFwUpdateMdReqReport(TRANSMIT_COMPLETE_FAIL);
        }
        return RECEIVED_FRAME_STATUS_SUCCESS;
      }
      return RECEIVED_FRAME_STATUS_FAIL;
      break;

    case FIRMWARE_UPDATE_ACTIVATION_SET_V5:
      if(false == Check_not_legal_response_job(rxOpt) &&
         (cmdLength == sizeof(ZW_FIRMWARE_UPDATE_ACTIVATION_SET_V5_FRAME)))
      {
        ZAF_TRANSPORT_TX_BUFFER  TxBuf;
        ZW_APPLICATION_TX_BUFFER* pTxBuf = &(TxBuf.appTxBuf);
        memset((uint8_t*)pTxBuf, 0, sizeof(ZW_APPLICATION_TX_BUFFER) );

        TRANSMIT_OPTIONS_TYPE_SINGLE_EX *pTxOptionsEx;
        RxToTxOptions(rxOpt, &pTxOptionsEx);

        memcpy(
            (uint8_t*)&pTxBuf->ZW_FirmwareUpdateActivationStatusReportV5Frame,
            (uint8_t*)&pCmd->ZW_FirmwareUpdateActivationSetV5Frame,
            sizeof(ZW_FIRMWARE_UPDATE_ACTIVATION_SET_V5_FRAME));

        pTxBuf->ZW_FirmwareUpdateActivationStatusReportV5Frame.cmdClass = COMMAND_CLASS_FIRMWARE_UPDATE_MD_V5;
        pTxBuf->ZW_FirmwareUpdateActivationStatusReportV5Frame.cmd = FIRMWARE_UPDATE_ACTIVATION_STATUS_REPORT_V5;
        pTxBuf->ZW_FirmwareUpdateActivationStatusReportV5Frame.firmwareUpdateStatus = ERROR_ACTIVATION_FIRMWARE;

        if(EQUEUENOTIFYING_STATUS_SUCCESS != Transport_SendResponseEP(
            (uint8_t *)pTxBuf,
            sizeof(ZW_FIRMWARE_UPDATE_ACTIVATION_STATUS_REPORT_V5_FRAME),
            pTxOptionsEx,
            NULL))
        {
          /*Job failed */
          ;
        }
        return RECEIVED_FRAME_STATUS_SUCCESS;
      }
      return RECEIVED_FRAME_STATUS_FAIL;
      break;
    case FIRMWARE_UPDATE_MD_PREPARE_GET_V5:
    {
      ZAF_TRANSPORT_TX_BUFFER  TxBuf;
      ZW_APPLICATION_TX_BUFFER* pTxBuf = &(TxBuf.appTxBuf);

      TRANSMIT_OPTIONS_TYPE_SINGLE_EX * pTxOptions = NULL;
      RxToTxOptions(rxOpt, &pTxOptions);

      pTxBuf->ZW_FirmwareUpdateMdPrepareReportV5Frame.cmdClass = COMMAND_CLASS_FIRMWARE_UPDATE_MD_V5;
      pTxBuf->ZW_FirmwareUpdateMdPrepareReportV5Frame.cmd = FIRMWARE_UPDATE_MD_PREPARE_REPORT_V5;

      pTxBuf->ZW_FirmwareUpdateMdPrepareReportV5Frame.status = FIRMWARE_UPDATE_MD_PREPARE_REPORT_NOT_DOWNLOADABLE_V5;

      // The firmware checksum is irrelevant when we report that it's not downloadable.
      pTxBuf->ZW_FirmwareUpdateMdPrepareReportV5Frame.firmwareChecksum1 = 0x00;
      pTxBuf->ZW_FirmwareUpdateMdPrepareReportV5Frame.firmwareChecksum2 = 0x00;

      if(EQUEUENOTIFYING_STATUS_SUCCESS != Transport_SendResponseEP(
          (uint8_t *)pTxBuf,
          sizeof(ZW_FIRMWARE_UPDATE_MD_PREPARE_REPORT_V5_FRAME),
          pTxOptions,
          NULL))
      {
        /*Job failed */
        ;
      }
      return RECEIVED_FRAME_STATUS_SUCCESS;
    }
      break;
    default:
      break;
  }
  return RECEIVED_FRAME_STATUS_NO_SUPPORT;
}

JOB_STATUS
CmdClassFirmwareUpdateMdStatusReport(
  RECEIVE_OPTIONS_TYPE_EX *rxOpt,
  uint8_t status,
  uint16_t waitTime,
  VOID_CALLBACKFUNC(pCbFunc)(TRANSMISSION_RESULT * pTransmissionResult))
{
  ZAF_TRANSPORT_TX_BUFFER  TxBuf;
  ZW_APPLICATION_TX_BUFFER *pTxBuf = &(TxBuf.appTxBuf);
  memset((uint8_t*)pTxBuf, 0, sizeof(ZW_APPLICATION_TX_BUFFER) );
  CommandClassSupervisionGetAdd(&(TxBuf.supervisionGet));

  TRANSMIT_OPTIONS_TYPE_SINGLE_EX* pTxOptionsEx = NULL;
  RxToTxOptions(rxOpt, &pTxOptionsEx);
  /* Send status, when finished */

  pTxBuf->ZW_FirmwareUpdateMdStatusReportV5Frame.cmdClass = COMMAND_CLASS_FIRMWARE_UPDATE_MD_V5;
  pTxBuf->ZW_FirmwareUpdateMdStatusReportV5Frame.cmd = FIRMWARE_UPDATE_MD_STATUS_REPORT_V5;
  pTxBuf->ZW_FirmwareUpdateMdStatusReportV5Frame.status = status;
  pTxBuf->ZW_FirmwareUpdateMdStatusReportV5Frame.waittime1 = (waitTime >> 8);
  pTxBuf->ZW_FirmwareUpdateMdStatusReportV5Frame.waittime2 = (waitTime & 0xff);
  if(EQUEUENOTIFYING_STATUS_SUCCESS != Transport_SendRequestEP(
      (uint8_t *)pTxBuf,
      sizeof(ZW_FIRMWARE_UPDATE_MD_STATUS_REPORT_V5_FRAME),
      pTxOptionsEx,
      pCbFunc))
  {
    TRANSMISSION_RESULT result = {0xff, TRANSMIT_COMPLETE_FAIL, TRANSMISSION_RESULT_FINISHED};
    pCbFunc(&result);
    return JOB_STATUS_BUSY;
  }
  return JOB_STATUS_SUCCESS;
}

JOB_STATUS
CmdClassFirmwareUpdateMdGet(
  RECEIVE_OPTIONS_TYPE_EX *rxOpt,
  uint16_t firmwareUpdateReportNumber)
{
  ZAF_TRANSPORT_TX_BUFFER  TxBuf;
  ZW_APPLICATION_TX_BUFFER *pTxBuf = &(TxBuf.appTxBuf);
  memset((uint8_t*)pTxBuf, 0, sizeof(ZW_APPLICATION_TX_BUFFER) );
  CommandClassSupervisionGetAdd(&(TxBuf.supervisionGet));

  TRANSMIT_OPTIONS_TYPE_SINGLE_EX* pTxOptionsEx = NULL;
  RxToTxOptions(rxOpt, &pTxOptionsEx);

  DPRINTF("CmdClassFirmwareUpdateMdGet %d ", firmwareUpdateReportNumber);

  /* Ask for the next report */
  pTxBuf->ZW_FirmwareUpdateMdGetV5Frame.cmdClass = COMMAND_CLASS_FIRMWARE_UPDATE_MD_V5;
  pTxBuf->ZW_FirmwareUpdateMdGetV5Frame.cmd = FIRMWARE_UPDATE_MD_GET_V5;
  pTxBuf->ZW_FirmwareUpdateMdGetV5Frame.numberOfReports = 1;
  pTxBuf->ZW_FirmwareUpdateMdGetV5Frame.properties1 = firmwareUpdateReportNumber >> 8;
  pTxBuf->ZW_FirmwareUpdateMdGetV5Frame.reportNumber2 = firmwareUpdateReportNumber & 0xFF;
  if(EQUEUENOTIFYING_STATUS_SUCCESS != Transport_SendRequestEP(
      (uint8_t *)pTxBuf,
      sizeof(ZW_FIRMWARE_UPDATE_MD_GET_V5_FRAME),
      pTxOptionsEx,
      NULL))
  {
    return JOB_STATUS_BUSY;
  }
  return JOB_STATUS_SUCCESS;
}

REGISTER_CC(COMMAND_CLASS_FIRMWARE_UPDATE_MD_V5, FIRMWARE_UPDATE_MD_VERSION_V5, handleCommandClassFWUpdate);
