/***************************************************************************
*
* @copyright 2018 Silicon Laboratories Inc.
* @brief ZWave+ Info Command Class source file
*/

/****************************************************************************/
/*                              INCLUDE FILES                               */
/****************************************************************************/

#include <CC_ZWavePlusInfo.h>
#include "config_app.h"
#include <ZW_plus_version.h>
#include <string.h>
#include <ZW_TransportEndpoint.h>

/****************************************************************************/
/*                      PRIVATE TYPES and DEFINITIONS                       */
/****************************************************************************/

typedef struct _CICON_
{
  ST_ENDPOINT_ICONS* pEpIcon;
  uint8_t nbrOfElements;
} CICON;

/****************************************************************************/
/*                              PRIVATE DATA                                */
/****************************************************************************/

CICON myIcon = {NULL, 0};

/****************************************************************************/
/*                              EXPORTED DATA                               */
/****************************************************************************/

// Nothing here.

/****************************************************************************/
/*                            PRIVATE FUNCTIONS                             */
/****************************************************************************/

void
CommandClassZWavePlusInfoInit(ST_ENDPOINT_ICONS* pEpIcon, uint8_t nbrOfElements)
{
  myIcon.pEpIcon = pEpIcon;
  myIcon.nbrOfElements = nbrOfElements;
}

received_frame_status_t
handleCommandClassZWavePlusInfo(
    RECEIVE_OPTIONS_TYPE_EX *rxOpt,
    ZW_APPLICATION_TX_BUFFER *pCmd,
    uint8_t cmdLength
)
{
  UNUSED(cmdLength);
  if (pCmd->ZW_Common.cmd == ZWAVEPLUS_INFO_GET)
  {
    /*Check pTxBuf is free*/
    if(false == Check_not_legal_response_job(rxOpt))
    {
      ZAF_TRANSPORT_TX_BUFFER  TxBuf;
      ZW_APPLICATION_TX_BUFFER *pTxBuf = &(TxBuf.appTxBuf);
      memset((uint8_t*)pTxBuf, 0, sizeof(ZW_APPLICATION_TX_BUFFER) );

      TRANSMIT_OPTIONS_TYPE_SINGLE_EX *pTxOptionsEx;
      RxToTxOptions(rxOpt, &pTxOptionsEx);
      pTxBuf->ZW_ZwaveplusInfoReportV2Frame.cmdClass = COMMAND_CLASS_ZWAVEPLUS_INFO;
      pTxBuf->ZW_ZwaveplusInfoReportV2Frame.cmd = ZWAVEPLUS_INFO_REPORT;
      pTxBuf->ZW_ZwaveplusInfoReportV2Frame.zWaveVersion = ZW_PLUS_VERSION;
      pTxBuf->ZW_ZwaveplusInfoReportV2Frame.roleType = APP_ROLE_TYPE;
      pTxBuf->ZW_ZwaveplusInfoReportV2Frame.nodeType = APP_NODE_TYPE;


      if((myIcon.nbrOfElements >= rxOpt->destNode.endpoint) && (0 != rxOpt->destNode.endpoint) && (NON_NULL(myIcon.pEpIcon)) )
      {
        pTxBuf->ZW_ZwaveplusInfoReportV2Frame.installerIconType1 = (myIcon.pEpIcon[rxOpt->destNode.endpoint-1].installerIconType >> 8);
        pTxBuf->ZW_ZwaveplusInfoReportV2Frame.installerIconType2 = (myIcon.pEpIcon[rxOpt->destNode.endpoint-1].installerIconType & 0xff);
        pTxBuf->ZW_ZwaveplusInfoReportV2Frame.userIconType1 = (myIcon.pEpIcon[rxOpt->destNode.endpoint-1].userIconType >> 8);
        pTxBuf->ZW_ZwaveplusInfoReportV2Frame.userIconType2 = (myIcon.pEpIcon[rxOpt->destNode.endpoint-1].userIconType & 0xff);
      }
      else
      {
        pTxBuf->ZW_ZwaveplusInfoReportV2Frame.installerIconType1 = (APP_ICON_TYPE >> 8);
        pTxBuf->ZW_ZwaveplusInfoReportV2Frame.installerIconType2 = (APP_ICON_TYPE & 0xff);
        pTxBuf->ZW_ZwaveplusInfoReportV2Frame.userIconType1 = (APP_USER_ICON_TYPE >> 8);
        pTxBuf->ZW_ZwaveplusInfoReportV2Frame.userIconType2 = (APP_USER_ICON_TYPE & 0xff);
      }

      {
        uint8_t txResult;

        txResult = Transport_SendResponseEP(
                      (uint8_t *)pTxBuf,
                      sizeof(pTxBuf->ZW_ZwaveplusInfoReportV2Frame),
                      pTxOptionsEx,
                      NULL);

        if (EQUEUENOTIFYING_STATUS_SUCCESS != txResult)
        {
          ;
        }
      }
      return RECEIVED_FRAME_STATUS_SUCCESS;
    }
    return RECEIVED_FRAME_STATUS_FAIL;
  }
  return RECEIVED_FRAME_STATUS_NO_SUPPORT;
}

REGISTER_CC(COMMAND_CLASS_ZWAVEPLUS_INFO, ZWAVEPLUS_INFO_VERSION_V2, handleCommandClassZWavePlusInfo);
